# Path to default location for backend when using local FileStore or ArtifactStore.
# Also used as default location for artifacts, when not provided, in non local file based backends
# (eg MySQL)
DEFAULT_LOCAL_FILE_AND_ARTIFACT_PATH = "./ruls"
SEARCH_MAX_RESULTS_DEFAULT = 1000
SEARCH_MAX_RESULTS_THRESHOLD = 50000



from abc import abstractmethod, ABCMeta


class AbstractStore:
    """
    Abstract class for Backend Storage.
    This class defines the API interface for front ends to connect with various types of backends.
    """

    __metaclass__ = ABCMeta

    def __init__(self):
        """
        Empty constructor for now. This is deliberately not marked as abstract, else every
        derived class would be forced to create one.
        """
        pass

    @abstractmethod
    def lc_create_run(self,
                      artifact):
        """
        Create a run under the specified experiment ID

        :param artifact: see LcArtifact in proto/life_cycle.proto

        :return: The ID of the created Run object
        """
        pass

    def create_tag(self, run_id, tag):
        """
        Set a tag for the specified run

        :param run_id: String id for the run
        :param tag: see /entities/lc_run_tag.py
        """
        pass




from Accuinsight.modeler.entities._modeler_object import _ModelerObject


class Alarm(_ModelerObject):
    """
        Deploy alarm parameter object.
    """
    def __init__(self):
        self._message = None
        self._notifiers = dict()
        self._obj_type = None

    @property
    def message(self):
        """alarm message to send"""
        return self._message

    @message.setter
    def message(self, message):
        self._message = message

    @property
    def notifiers(self):
        """alarm notifier list"""
        return self._notifiers

    @notifiers.setter
    def notifiers(self, notifiers):
        self._notifiers = notifiers

    @property
    def obj_type(self):
        """alarm object type"""
        return self._obj_type

    @obj_type.setter
    def obj_(self, obj_type):
        self._obj_type = obj_type

    @classmethod
    def from_proto(cls, proto):
        pass

    def get_alarm_param(self):
        """Create and return backend API parameter"""
        import json

        if self.message is None:
            raise Exception("message cannot be None")

        data = dict()

        data['message'] = self.message
        data['notifiers'] = self.notifiers
        data['type'] = self.type

        return json.dumps(data)



import uuid
import datetime


def get_run_id():
    return str(uuid.uuid4()).upper()


class get_time(object):
    @staticmethod
    def now():
        return datetime.datetime.now()

    @staticmethod
    def logging_time():
        dt = datetime.datetime.now()
        return dt.strftime('%Y-%m-%d %H:%M:%S')



from Accuinsight.modeler.core.LcConst.LcConst import RUN_OBJ_DATA_VERSION
from Accuinsight.modeler.protos.life_cycle_pb2 import LcParam


def get_data_version(param_data):
    data_version = ''
    if RUN_OBJ_DATA_VERSION in param_data:
        data_version = param_data[RUN_OBJ_DATA_VERSION]

    return LcParam(
        key=RUN_OBJ_DATA_VERSION,
        value=data_version
    )



from Accuinsight.modeler.entities._modeler_object import _ModelerObject
from Accuinsight.modeler.protos.life_cycle_pb2 import LcArtifact as ProtoLcArtifact


class LcArtifact(_ModelerObject):
    """
    Parameter object.
    """

    def __init__(self, name, version):
        self._name = name
        self._version = version

    @property
    def name(self):
        """String name corresponding to the artifact."""
        return self._name

    @property
    def version(self):
        """String version of the artifact."""
        return self._version

    def to_proto(self):
        param = ProtoLcArtifact()
        param.name = self.name
        param.version = self.version
        return param

    @classmethod
    def from_proto(cls, proto):
        return cls(proto.name, proto.version)
